@@ -1,8 +1,8 @@
 #!/usr/bin/perl -w
 #
-# Copyright (C) 2003, 2004  Pascal Giard
+# Copyright (C) 2003, 2004, 2007  Pascal Giard
 #
-# Author:             Pascal Giard <evilynux@yahoo.com>
+# Author:             Pascal Giard <evilynux@gmail.com>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -45,11 +45,11 @@ use vars qw($NAME %options $xml_root $child_pid $mkisocmd
             $diag_feedback $diag_properties
             $statusbar $installed_emovix $tmpfolder
             $destpath $srcpath $help $destination $srcdir $current_path
-            @boot_help_langs @kblayouts @fontsets $bootlabels
-            @extra_mkisofs_options $extra_mkisofs_options
+            @boot_help_langs @menu_langs @remotes @kblayouts @fontsets
+            $bootlabels @extra_genisoimage_options $extra_genisoimage_options
             @extra_mplayer_options $extra_mplayer_options
             @unwanted_mplayer_options $unwanted_mplayer_options
-            $filelist @real_filenames
+            $filelist @real_filenames $codeclist @real_codecfilenames
            );
 
 $NAME="MoviXMaker-2";
@@ -60,7 +60,7 @@ GetOptions (
 	    "tmpDir=s"           => \$options{'tmpdir'},
  	    "output-file=s"       => \$destpath,
  	    "boot-label=s"        => \$options{'bootlabel'},
- 	    "extra-mkisofs-options=s"     => \@extra_mkisofs_options,
+ 	    "extra-genisoimage-options=s"     => \@extra_genisoimage_options,
  	    "extra-mplayer-options=s"     => \@extra_mplayer_options,
  	    "unwanted-mplayer-options=s"  => \@unwanted_mplayer_options,
 	    # 	    "make-empty-iso"      => \$make_empty_iso,
@@ -210,8 +210,8 @@ sub read_config {
     if (!defined $options{'nodma'});
 
   # Other options
-  @extra_mkisofs_options = $cfg->val('Others','extra_mkisofs_options')
-    if (!defined @extra_mkisofs_options);
+  @extra_genisoimage_options = $cfg->val('Others','extra_genisoimage_options')
+    if (!defined @extra_genisoimage_options);
   @extra_mplayer_options = $cfg->val('Others','extra_mplayer_options')
     if (!defined @extra_mplayer_options);
   @unwanted_mplayer_options = $cfg->val('Others','unwanted_mplayer_options')
@@ -250,8 +250,9 @@ sub on_button_create_released {
   $button->set_sensitive(FALSE);
   $destination->set_sensitive(FALSE);
   $srcdir->set_sensitive(FALSE);
-  $xml_root->get_widget ("cb_nocustomize")->set_sensitive(FALSE);
-  $xml_root->get_widget ("cb_noaddfiles")->set_sensitive(FALSE);
+  $xml_root->get_widget ("cb_customize")->set_sensitive(FALSE);
+  $xml_root->get_widget ("cb_addfiles")->set_sensitive(FALSE);
+  $xml_root->get_widget ("cb_addcodecs")->set_sensitive(FALSE);
   $statusbar->set_status("Proceeding...");
 
   prepare_creation();
@@ -259,14 +260,15 @@ sub on_button_create_released {
 
 sub reenable {
   $xml_root->get_widget ("button_create")->set_sensitive(TRUE);
-  $xml_root->get_widget ("cb_nocustomize")->set_sensitive(TRUE);
-  $xml_root->get_widget ("cb_noaddfiles")->set_sensitive(TRUE);
+  $xml_root->get_widget ("cb_customize")->set_sensitive(TRUE);
+  $xml_root->get_widget ("cb_addfiles")->set_sensitive(TRUE);
+  $xml_root->get_widget ("cb_addcodecs")->set_sensitive(TRUE);
   $destination->set_sensitive(TRUE);
   $srcdir->set_sensitive(TRUE);
 }
 
 
-# The verbose_mkisofs_output window gets closed by the user
+# The verbose_genisoimage_output window gets closed by the user
 sub on_fbbutton_released {
   $diag_feedback->hide();
   $statusbar->set_status("$NAME $config::version");
@@ -292,12 +294,33 @@ sub on_add_file {
   $widget->hide();
 }
 
-# The user is adding a file to the ISO
+# The user is adding a codec to the ISO
+sub on_add_codec {
+  my ( $ok_button, $widget, $undef ) = @_;
+
+  foreach my $filename ( $widget->get_selections() ) {
+    push( @{$codeclist->{'data'}}, $filename );
+    push @real_codecfilenames, $filename;
+    print "Adding codec: $filename\n"
+      if ($options{'debug'});
+  }
+
+  $widget->hide();
+}
+
+# The user is removing a file from the ISO
 sub on_remove_file {
   map( { delete( $filelist->{'data'}[$_] );
 	 delete( $real_filenames[$_] ) }
        reverse( $filelist->get_selected_indices() ) );
 }
+
+# The user is removing a codec from the ISO
+sub on_remove_codec {
+  map( { delete( $codeclist->{'data'}[$_] );
+	 delete( $real_codecfilenames[$_] ) }
+       reverse( $codeclist->get_selected_indices() ) );
+}
 ### end callbacks
 
 ###############################
@@ -343,7 +366,7 @@ sub update_options {
     set_text($options{'boot_help_lang'});
 
   ##
-  # *** FIXME: support for menu_lang should be added
+  # *** FIXME: support for menu_lang, remote and tv_region should be added
   ##
 
   # Fonsets
@@ -381,7 +404,7 @@ sub set_options {
     # Debug mode.
     $xml_root->get_widget ("cb_debug")->set_active($options{'debug'});
 
-    # Verbose mkisofs output.
+    # Verbose genisoimage output.
     $xml_root->get_widget ("cb_verbose")->set_active($options{'verbose'});
 
     # Donnot delete tmp directory
@@ -404,13 +427,13 @@ sub set_options {
     unless (defined $options{'nogui'});
 
   #######
-  ## mkisofs options
+  ## genisoimage options
   #######
-  if (@extra_mkisofs_options > 0) {
-    $extra_mkisofs_options =
-      join(' ',split(/,/,join(',',@extra_mkisofs_options)));
-    $xml_root->get_widget ("entry_extra_mkisofs_options")->
-      set_text($extra_mkisofs_options) unless (defined $options{'nogui'});;
+  if (@extra_genisoimage_options > 0) {
+    $extra_genisoimage_options =
+      join(' ',split(/,/,join(',',@extra_genisoimage_options)));
+    $xml_root->get_widget ("entry_extra_genisoimage_options")->
+      set_text($extra_genisoimage_options) unless (defined $options{'nogui'});;
   }
   #######
   ## mplayer options
@@ -479,7 +502,7 @@ sub set_options {
     }
 
     ##
-    # *** FIXME : menu_lang needs to be handled.
+    # *** FIXME : menu_lang, remote and tv_region need to be handled.
     ##
 
     # Default keyboard layout.
@@ -535,7 +558,7 @@ sub get_options {
     $options{'debug'} =
       $xml_root->get_widget ("cb_debug")->get_active || 0;
 
-    # Verbose mkisofs output.
+    # Verbose genisoimage output.
     $options{'verbose'} = $xml_root->get_widget ("cb_verbose")->get_active || 0;
 
     # Donnot delete tmp dir.
@@ -548,11 +571,11 @@ sub get_options {
   $options{'tmpdir'} =~ s/([^\/])$/$1\// ;
 
   #######
-  ## mkisofs options
+  ## genisoimage options
   #######
   if (!defined $options{'nogui'}) {
-    $extra_mkisofs_options =
-      $xml_root->get_widget ("entry_extra_mkisofs_options")->get_text;
+    $extra_genisoimage_options =
+      $xml_root->get_widget ("entry_extra_genisoimage_options")->get_text;
     #######
     ## mplayer options
     #######
@@ -619,7 +642,7 @@ sub get_options {
 DEBUG = $options{'debug'}
 VERBOSE = $options{'verbose'}
 NODELETE = $options{'nodelete'}
-EXTRA_MKISOFS_OPTIONS = "$extra_mkisofs_options"
+EXTRA_GENISOIMAGE_OPTIONS = "$extra_genisoimage_options"
 EXTRA_MPLAYER_OPTIONS = "$extra_mplayer_options"
 UNWANTED_MPLAYER_OPTIONS = "$unwanted_mplayer_options"
 TMPDIR = "$options{'tmpdir'}"
@@ -686,10 +709,26 @@ sub prepare_interface {
   map { $_->set_resizable (TRUE) } $filelist->get_columns;
 
   # packed into a scrolled window...
-  my $scrolled = $xml_root->get_widget ("scrolledwindow_addfiles");
-  $scrolled->add ($filelist);
+  my $scrolled_files = $xml_root->get_widget ("scrolledwindow_addfiles");
+  $scrolled_files->add ($filelist);
   # all in one go
-  $scrolled->show_all;
+  $scrolled_files->show_all;
+
+  # Codeclist creation (codecs added to the dics image)
+  $codeclist =
+    Gtk2::SimpleList->new ('Path to codec file or directory' => 'text');
+
+  # Gtk2::SimpleList is derived from Gtk2::TreeView, so you can
+  # do anything you'd do to a treeview.
+  $codeclist->set_rules_hint (TRUE);
+  $codeclist->get_selection->set_mode ('multiple');
+  map { $_->set_resizable (TRUE) } $codeclist->get_columns;
+
+  # packed into a scrolled window...
+  my $scrolled_codecs = $xml_root->get_widget ("scrolledwindow_addcodecs");
+  $scrolled_codecs->add ($codeclist);
+  # all in one go
+  $scrolled_codecs->show_all;
 
   #######################
   # Options specified at command line
@@ -736,7 +775,7 @@ sub warning {
 
 
 sub create_iso {
-  use vars qw($progress $diag_progress $fbtext $fbtextbuffer $output $coutput);
+  my ($progress, $diag_progress, $fbtext, $fbtextbuffer, $output, $coutput);
 
   # Create temp dir and update the global variable
   $tmpfolder = create_tmp_folder();
@@ -750,7 +789,7 @@ sub create_iso {
   unless (defined $options{'nogui'}) {
     $fbtext = $xml_root->get_widget ("fbtext");
 
-    # Verbose mkisofs in dialog if enabled
+    # Verbose genisoimage in dialog if enabled
     if ($options{'verbose'}) {
       $fbtextbuffer = $fbtext->get_buffer;
       $fbtextbuffer->delete($fbtextbuffer->get_iter_at_offset(0),
@@ -758,7 +797,7 @@ sub create_iso {
       $diag_feedback->show_all();
     }
 
-    $statusbar->set_status("Calling mkisofs...");
+    $statusbar->set_status("Calling genisoimage...");
   }
 
   my $isolinux = ($movixversion =~ /^eMoviX/) ?
@@ -773,15 +812,15 @@ sub create_iso {
     "-b ${isolinux}isolinux.bin -c ${isolinux}isolinux.boot ".
       "-sort ${isolinux}iso.sort";
   ###############
-  ### Standard mkisofs options: -v=>verbose, -r=>rock-ridge,
+  ### Standard genisoimage options: -v=>verbose, -r=>rock-ridge,
   ####                          -J=>Joliet,  -f=>follow symbolic links
   ##############
   my $stdOptions = "-v -r -J -f";
   #########################################
 
-  $child_pid = open(CMDEXEC,"cd $tmpfolder ; mkisofs -o $destpath $stdOptions ".
-		    "-V '$movixversion' $extra_mkisofs_options $bootOptions ".
-		    "-A '$movixversion' . 2>&1 |");
+  $child_pid = open(CMDEXEC,"cd $tmpfolder ; genisoimage -o $destpath ".
+                    "$stdOptions -V '$movixversion' $extra_genisoimage_options ".
+                    "$bootOptions -A '$movixversion' . 2>&1 |");
   print STDERR "CHILD_PID : $child_pid\n" if ($options{'debug'});
 
   unless (defined $options{'nogui'}) {
@@ -801,12 +840,12 @@ sub create_iso {
     } elsif ( $_ =~ /(writing|scanning)/i and (!defined $options{'nogui'})) {
       $diag_progress->set_title($1);
       $statusbar->set_status($1);
-    } elsif ( $_ =~ /mkisofs:\sWarning:\s(.*)/ ) {
+    } elsif ( $_ =~ /genisoimage:\sWarning:\s(.*)/ ) {
       print STDERR "WARNING : $1\n"
 	if ($options{'debug'} || (defined $options{'nogui'}));
       ######
-      # Detecting mkisofs errors and rejecting Warnings
-    } elsif ( $_ =~ /mkisofs:(.*)/ ) {
+      # Detecting genisoimage errors and rejecting Warnings
+    } elsif ( $_ =~ /genisoimage:(.*)/ ) {
       $diag_progress->hide unless (defined $options{'nogui'});
       error($1);
     }
@@ -838,7 +877,7 @@ sub create_iso {
       reenable();
     }
     ######
-    # Special treatment for mkisofs --help
+    # Special treatment for genisoimage --help
   } elsif ( $coutput =~ /Usage:\s(.*)/ ) {
     $coutput =~ s/^.*Warning.*\n//; # We don't want the warnings to be displayed
     if (defined $options{'nogui'}) {
@@ -866,9 +905,6 @@ sub get_movix_version {
   if ($installed_emovix) {
 
     $movixversion = "eMoviX-" . `echo -n \$(movix-version 2>/dev/null)`;
-    # Desactivate the menu language selection for installed eMoviX
-    $xml_root->get_widget ("optionmenu_menu_lang")->set_sensitive(FALSE)
-      unless (defined $options{'nogui'});
 
     $statusbar->set_status("Detected $movixversion.")
       unless (defined $options{'nogui'});
@@ -943,6 +979,48 @@ sub get_boot_help_languages {
   return @files;
 }
 
+# Tries to detect the available menu languages
+sub get_menu_languages {
+  return @boot_help_langs if ( $movixversion =~ /^eMoviX/ );
+
+  my $dir = "${srcpath}src/movix";
+
+  opendir (LP, $dir);
+  my @files = grep { /^menu\.conf\.([a-z]{2,3})$/ } readdir(LP);
+  closedir LP;
+
+  # We only want a list of the available languages
+  if (@files) {
+    foreach (@files) {
+      $_ =~ s/^menu\.conf\.//;
+    }
+  }
+
+  @files = qw(en) unless (@files);
+
+  return @files;
+}
+
+# Tries to detect the available menu languages
+sub get_remotes {
+
+  my $dir = ($installed_emovix) ?
+    "${srcpath}remotes" : "${srcpath}src/movix/remotes";
+
+  opendir (LP, $dir);
+  my @files = grep { /^lircrc\.([a-z]+)$/ } readdir(LP);
+  closedir LP;
+
+  # We only want a list of the available languages
+  if (@files) {
+    foreach (@files) {
+      $_ =~ s/^lircrc\.//;
+    }
+  }
+
+  return @files;
+}
+
 # Detects the available keyboard layouts
 sub get_kblayouts {
   my $dir = "${srcpath}keyboard-i18n";
@@ -990,6 +1068,24 @@ sub validate_boot_help_language {
   return "en" unless $choosen_lang;
 }
 
+sub validate_menu_language {
+  my $choosen_lang;
+  foreach my $lang (@menu_langs) {
+    $choosen_lang = $options{'menu_lang'}
+      if ($lang =~ /$options{'menu_lang'}/);
+  }
+  return "en" unless $choosen_lang;
+}
+
+sub validate_remote {
+  my $choosen_remote;
+  foreach my $remote (@remotes) {
+    $choosen_remote = $options{'remote'}
+      if ($remote =~ /$options{'remote'}/);
+  }
+  return "none" unless $choosen_remote;
+}
+
 sub validate_kblayout {
   my $choosen_kblayout;
   foreach my $kblayout (@kblayouts) {
@@ -1019,10 +1115,14 @@ sub validate_bootlabel {
 
 sub get_movix_options {
   @boot_help_langs = get_boot_help_languages;
+  @menu_langs = get_menu_languages;
+  @remotes = get_remotes;
   @kblayouts = get_kblayouts;
   @fontsets = get_fontsets;
   $bootlabels = get_bootlabels();
   $options{'boot_help_lang'} = validate_boot_help_language();
+  $options{'menu_lang'} = validate_menu_language();
+  $options{'remote'} = validate_remote();
   $options{'kblayout'} = validate_kblayout();
   $options{'fontset'} = validate_fontset();
   $options{'bootlabel'} = validate_bootlabel();
@@ -1033,6 +1133,16 @@ sub get_movix_options {
       print STDERR "$file ";
     }
     print STDERR ".\n";
+    print STDERR "FOUND MENU LANGUAGE(S) : ";
+    foreach my $file (@menu_langs) {
+      print STDERR "$file ";
+    }
+    print STDERR ".\n";
+    print STDERR "FOUND REMOTE(S) : ";
+    foreach my $file (@remotes) {
+      print STDERR "$file ";
+    }
+    print STDERR ".\n";
     print STDERR "FOUND KBLAYOUT(S) : ";
     foreach my $file (@kblayouts) {
       print STDERR "$file ";
@@ -1115,31 +1225,53 @@ sub prepare_creation {
   # Update the interface with avail. options
   update_options();
 
-  # Did the user toggled the "Do not customize disc image" checkbox ?
-  if ($xml_root->get_widget ("cb_nocustomize")->get_active
-      and $xml_root->get_widget ("cb_noaddfiles")->get_active) {
+  ### FIXME: Very ugly dialog handling ###
+  if ($xml_root->get_widget ("cb_addfiles")->get_active
+      or $xml_root->get_widget ("cb_addcodecs")->get_active) {
+    $xml_root->get_widget ("btn_customize_next")->show;
+    $xml_root->get_widget ("btn_customize_ok")->hide;
+  }
+
+  if ($xml_root->get_widget ("cb_addfiles")->get_active) {
+      if ($xml_root->get_widget ("cb_addcodecs")->get_active) {
+	$xml_root->get_widget ("btn_addfiles_next")->show;
+      } else {
+	$xml_root->get_widget ("btn_addfiles_ok")->show;
+      }
+  }
+
+  if ($xml_root->get_widget ("cb_addcodecs")->get_active) {
+    $xml_root->get_widget ("btn_addfiles_next")->show;
+    $xml_root->get_widget ("btn_addfiles_ok")->hide;
+
+    if ($xml_root->get_widget ("cb_customize")->get_active) {
+      $xml_root->get_widget ("btn_addfiles_previous")->show;
+    } else {
+      $xml_root->get_widget ("btn_addfiles_previous")->hide;
+    }
+  }
+
+  # Does the user want to customize the disc image ?
+  if (!$xml_root->get_widget ("cb_customize")->get_active
+      and !$xml_root->get_widget ("cb_addfiles")->get_active
+      and !$xml_root->get_widget ("cb_addcodecs")->get_active) {
     # Directly creating the image, no customization
     create_iso();
+    return;
 
-    # User choose only to customize
-  } elsif ($xml_root->get_widget ("cb_noaddfiles")->get_active) {
-    $xml_root->get_widget ("btn_customize_next")->hide;
-    $xml_root->get_widget ("btn_customize_ok")->show;
+  # Show dialog to customize disc image
+  } elsif ($xml_root->get_widget ("cb_customize")->get_active){
     $xml_root->get_widget ("diag_customize")->show();
 
-    # User choose only to add files
-  } elsif ($xml_root->get_widget ("cb_nocustomize")->get_active) {
-    $xml_root->get_widget ("btn_addfiles_previous")->hide;
+  # Show dialog to add files to the disc image
+  } elsif ($xml_root->get_widget ("cb_addfiles")->get_active){
     $xml_root->get_widget ("diag_addfiles")->show();
 
-    # User choose to customize disc image and add files
-  } else {
-    # Let's choose the options
-    $xml_root->get_widget ("btn_customize_ok")->hide;
-    $xml_root->get_widget ("btn_customize_next")->show;
-    $xml_root->get_widget ("btn_addfiles_previous")->show;
-    $xml_root->get_widget ("diag_customize")->show();
+  # Show dialog to add codecs to the disc image
+  } elsif ($xml_root->get_widget ("cb_addcodecs")->get_active){
+    $xml_root->get_widget ("diag_addcodecs")->show();
   }
+  ### END OF FIXME: Very ugly dialog handling ###
 
 }
 
@@ -1155,6 +1287,7 @@ sub create_tmp_folder {
     "boot" : "isolinux";
 
   my $tmpDir = "${options{'tmpdir'}}movix-$date";
+  print "Temporary directory: $tmpDir\n" if ($options{'debug'});
 
   mkdir  "$tmpDir";
 
@@ -1178,6 +1311,7 @@ sub create_tmp_folder {
     chdir "eMoviX";
   }
 
+  # Adding user specified multimedia files
   foreach my $filename (@real_filenames) {
     system "ln -s $filename ." if ($filename);
   }
@@ -1267,6 +1401,20 @@ sub create_tmp_folder {
     }
   }
 
+
+  if ( -e $srcpath . "mplayer/codecs") {
+    system "rm -f mplayer/codecs" and mkdir -p "mplayer/codecs"
+      and system "ln -s " . $srcpath . "mplayer/codecs/* mplayer/codecs";
+  }
+
+  if (! -e "mplayer/codecs") {
+    system "mkdir -p mplayer/codecs";
+
+    foreach my $filename (@real_codecfilenames) {
+      system "ln -s $filename mplayer/codecs" if ($filename);
+    }
+  }
+
   create_isolinuxcfg($isolinux);
 
   # Creating the file containing the default [e]MoviX[2] boot parameters
@@ -1312,6 +1460,8 @@ sub create_isolinuxcfg {
 sub create_bootrc {
   my $file = shift;
 
+  system "rm -f $file" if( -e $file );
+
   open BOOTRC, "> " . $file;
 
   print BOOTRC "EXTRA-MPLAYER-OPTIONS=$extra_mplayer_options\n"
@@ -1354,7 +1504,7 @@ Options:
 
  Generic:
 
-   --verbose               Verbose mkisofs output
+   --verbose               Verbose genisoimage output
    --debug                 Debug mode (kind of movixmaker verbose)
    --version               Displays MoviXMaker-2's version and exits
    --nogui                 Do not show the graphical interface
@@ -1381,8 +1531,8 @@ Options:
    --eject                 Make MoviX eject the CD once it finished playing
    --reboot                Make MoviX reboot the PC once it finished playing
    --shut                  Make MoviX shut the PC off once it finished playing
-   --extra-mkisofs-options=\"opt1 opt2 ...\"
-                           Extra options for mkisofs
+   --extra-genisoimage-options=\"opt1 opt2 ...\"
+                           Extra options for genisoimage
    --extra-mplayer-options=\"opt1 opt2 ...\"
                            Extra options for MPlayer
    --unwanted-mplayer-options=\"opt1 opt2 ...\"
@@ -1408,7 +1558,7 @@ MoviXMaker-2 - Easily create customized eMoviX/MoviX/MoviX2 disc images
                   [-o=OutputFile] [-t=TempDir] [-b=BootLabel]
                   [--loop=NbrOfLoops] [--random] [--nodma]
                   [--eject] [--reboot] [--shut]
-                  [--extra-mkisofs-options="opt1 opt2 ..."]
+                  [--extra-genisoimage-options="opt1 opt2 ..."]
                   [--extra-mplayer-options="opt1 opt2 ..."]
                   [--unwanted-mplayer-options="opt1 opt2 ..."]
 
@@ -1450,7 +1600,7 @@ Display help messages [similar to this help] then exits.
 
 =over
 
-Verbose mkisofs output.
+Verbose genisoimage output.
 
 =back
 
@@ -1608,11 +1758,11 @@ Make *MoviX* poweroff the PC once it finished playing.
 
 =back
 
-=head2 --extra-mkisofs-options="opt1 opt2 ..."
+=head2 --extra-genisoimage-options="opt1 opt2 ..."
 
 =over
 
-Extra options for mkisofs.
+Extra options for genisoimage.
 
 =back
 
